/***************************************************************************
                          kplayer.h
                          ---------
    begin                : Sat Nov 16 10:12:50 EST 2002
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef KPLAYER_H
#define KPLAYER_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kaction.h>
#include <kmainwindow.h>
#include <kuniqueapplication.h>

#include "kplayerprocess.h"
#include "kplayerwidget.h"

class KCmdLineArgs;
class KPlayerPopupSliderAction;
class KPlayerSliderAction;
class KRecentFilesAction;
class KStatusBarLabel;
class KToggleAction;
class KURL;

#define KPLAYER_TOOLBARS 7
#define FIRST_VIDEO_TOOLBAR 3

/** The KPlayer main window class.
  *@author kiriuja
  */
class KPlayer : public KMainWindow
{
  Q_OBJECT 

  typedef struct _Toolbar
  {
    const char* name;
    const char* action;
    bool normally_visible;
    bool fullscreen_visible;
  } Toolbar;

public:
  /** Constructs the KPlayer main window. Parameters are passed to KMainWindow constructor.
    */
  KPlayer (QWidget* parent=0, const char *name=0);
  /** KPlayer main window destructor. Does nothing.
    */
  virtual ~KPlayer() { }

  /** Returns the KPlayerProcess object.
    */
  KPlayerProcess* kPlayerProcess (void) const
    { return kPlayerWidget() -> kPlayerProcess(); }
  /** Returns the KPlayerWidget object.
    */
  KPlayerWidget* kPlayerWidget (void) const
    { return kPlayerWorkspace() -> kPlayerWidget(); }
  /** Returns the KPlayerWorkspace object.
    */
  KPlayerWorkspace* kPlayerWorkspace (void) const
    { return (KPlayerWorkspace*) centralWidget(); }

  /** Starts playing a new file.
    */
  void start (void);
  /** Starts playback.
    */
  void play (void);
  /** Zooms the video to the correct size.
    */
  void zoom (void);

protected:
  /** Initializes the KActions of the application.
    */
  void initActions (void);
  /** Sets up the status bar.
    */
  void initStatusBar (void);

  /** Save general options like all bar positions and status as well as the geometry
      and the recent file list to the configuration file. */
  void saveOptions (void);
  /** Read general options again and initialize all variables like the recent file list.
    */
  void readOptions (void);
  /** Saves the current URLs during session end to the session config file.
      @see KTMainWindow#saveProperties */
  virtual void saveProperties (KConfig*) const;
  /** Restores the URLs from the session config file.
      @see KTMainWindow#readProperties */
  virtual void readProperties (KConfig*);

  /** Enables or disables player actions.
    */
  void enablePlayerActions (void);
  /** Enables or disables video actions.
    */
  void enableVideoActions (void);
  /** Enables or disables, checks or unchecks zoom actions.
    */
  void enableZoomActions (void);
  /** Checks or unchecks bar actions. */
//void checkBarActions (void);
  /** Enables a toolbar.
   * @param index the toolbar number
   */
  void enableToolbar (int index);
  /** Disables a toolbar.
   * @param index the toolbar number
   */
  void disableToolbar (int index);

  /** Sets the sound volume.
    */
  void setVolume (int);
  /** Sets the video contrast.
    */
  void setContrast (int);
  /** Sets the video brightness.
    */
  void setBrightness (int);
  /** Sets the video hue.
    */
  void setHue (int);
  /** Sets the video saturation.
    */
  void setSaturation (int);

  /** Changes the statusbar contents for the standard label permanently, used to indicate current actions.
   */
  void setStatusText (const QString&);

  /** Processes the widget events. Passes them on to KMainWindow. */
//virtual bool event (QEvent*);
  /** Handles main window resizing. */
//virtual void resizeEvent (QResizeEvent*);
  /** Displays the right click popup menu.
    */
  virtual void contextMenuEvent (QContextMenuEvent*);

  /** Retrieves an action from the actionCollection by name.
    */
  KAction* action (const char* name) const
    { return actionCollection() -> action (name); }
  /** Retrieves a slider action from the actionCollection by name.
    */
  KPlayerSliderAction* sliderAction (const char* name) const
    { return (KPlayerSliderAction*) action (name); }
  /** Retrieves a slider action from the actionCollection by name.
    */
  KPlayerPopupSliderAction* popupAction (const char* name) const
    { return (KPlayerPopupSliderAction*) action (name); }
  /** Retrieves a toggle action from the actionCollection by name.
    */
  KToggleAction* toggleAction (const char* name) const
    { return (KToggleAction*) action (name); }
  /** Retrieves the recent files action from the actionCollection.
    */
  KRecentFilesAction* recentFilesAction (void) const
    { return (KRecentFilesAction*) action (KStdAction::stdName (KStdAction::OpenRecent)); }

  // Following should be private.
  bool m_resizing, m_updating, m_initial_fullscreen;
  bool m_menubar_normally_visible, m_statusbar_normally_visible;
  bool m_menubar_fullscreen_visible, m_statusbar_fullscreen_visible;
  Toolbar m_toolbar [KPLAYER_TOOLBARS];
  QSize m_previous_size, m_video_size;
  KStatusBarLabel *m_status_label, *m_state_label, *m_progress_label;

public slots:
  /** Stops the KPlayerProcess and closes the KPlayerWidget.
    */
  virtual void closeEvent (QCloseEvent*);

  /** Displays the Open File dialog and starts playing the chosen file.
    */
  void onFileOpen (void);
  /** Displays the Open URL dialog and starts playing from the entered URL.
    */
  void onFileOpenUrl (void);
  /** Displays the Open Subtitle File dialog and loads the chosen subtitles.
      Restarts playback if a file was playing.
    */
  void onFileOpenSubtitle (void);
  /** Displays the Open Subtitle URL dialog and loads the chosen subtitles.
      Restarts playback if a file was playing.
    */
  void onFileOpenSubtitleUrl (void);
  /** Unloads subtitles and prevents subtitle autoloading.
      Restarts playback if a file was playing.
    */
  void onFileUnloadSubtitles (void);
  /** Opens a file from the recent files menu.
    */
  void onFileOpenRecent (const KURL&);
  /** Ends the program by closing the main window.
    */
  void onFileQuit (void);

  /** Toggles full screen mode.
    */
  void onViewFullScreen (void);
  /** Scales video up.
    */
  void onViewZoomIn (void);
  /** Scales video down.
    */
  void onViewZoomOut (void);
  /** Scales video to 50%.
    */
  void onViewZoom12 (void);
  /** Scales video to 100%.
    */
  void onViewZoom11 (void);
  /** Scales video to 150%.
    */
  void onViewZoom32 (void);
  /** Scales video to 200%.
    */
  void onViewZoom21 (void);
  /** Scales video to 250%.
    */
  void onViewZoom52 (void);
  /** Scales video to 300%.
    */
  void onViewZoom31 (void);
  /** Toggles the option of maintaining the original video aspect ratio.
    */
  void onViewMaintainOriginalAspect (void);
  /** Toggles the option of maintaining the current video aspect ratio.
    */
  void onViewMaintainCurrentAspect (void);
  /** Forces 4 to 3 video aspect ratio.
    */
  void onViewAspect43 (void);
  /** Forces 16 to 9 video aspect ratio.
    */
  void onViewAspect169 (void);

  /** Plays the currently loaded file.
    */
  void onPlayerPlay (void);
  /** Pauses the player.
    */
  void onPlayerPause (void);
  /** Stops the player.
    */
  void onPlayerStop (void);
  /** Moves ten seconds forward.
    */
  void onPlayerForward (void);
  /** Moves one minute forward.
    */
  void onPlayerFastForward (void);
  /** Moves ten seconds backward.
    */
  void onPlayerBackward (void);
  /** Moves one minute backward.
    */
  void onPlayerFastBackward (void);
  /** Moves to the beginning.
    */
  void onPlayerStart (void);
  /** Increases the volume.
    */
  void onPlayerVolumeUp (void);
  /** Decreases the volume.
    */
  void onPlayerVolumeDown (void);
  /** Toggles audio playback.
    */
  void onPlayerMute (void);
  /** Toggle the soft frame dropping option.
    */
  void onPlayerSoftFrameDrop (void);
  /** Toggle the hard frame dropping option.
    */
  void onPlayerHardFrameDrop (void);

  /** Increases the video brightness.
    */
  void onVideoBrightnessUp (void);
  /** Decreases the video brightness.
    */
  void onVideoBrightnessDown (void);
  /** Increases the video contrast.
    */
  void onVideoContrastUp (void);
  /** Decreases the video contrast.
    */
  void onVideoContrastDown (void);
  /** Increases the video hue.
    */
  void onVideoHueUp (void);
  /** Decreases the video hue.
    */
  void onVideoHueDown (void);
  /** Increases the video saturation.
    */
  void onVideoSaturationUp (void);
  /** Decreases the video saturation.
    */
  void onVideoSaturationDown (void);

  /** Toggles the menu bar.
    */
  void onViewMenubar (void);
  /** Toggles the main toolbar.
    */
  void onViewMainToolbar (void);
  /** Toggles the progress toolbar.
    */
  void onViewProgressToolbar (void);
  /** Toggles the volume toolbar.
    */
  void onViewVolumeToolbar (void);
  /** Toggles the brightness toolbar.
    */
  void onViewBrightnessToolbar (void);
  /** Toggles the contrast toolbar.
    */
  void onViewContrastToolbar (void);
  /** Toggles the hue toolbar.
    */
  void onViewHueToolbar (void);
  /** Toggles the Saturation toolbar.
    */
  void onViewSaturationToolbar (void);
  /** Toggles the status bar.
    */
  void onViewStatusbar (void);

  /** Moves to the position chosen.
    */
  void onProgressChanged (int);
  /** Changes the volume level.
    */
  void onVolumeChanged (int);
  /** Changes the video brightness.
    */
  void onBrightnessChanged (int);
  /** Changes the video contrast.
    */
  void onContrastChanged (int);
  /** Changes the video hue.
    */
  void onHueChanged (int);
  /** Changes the video saturation.
    */
  void onSaturationChanged (int);

  /** Receives the stateChanged signal from KPlayerProcess.
    */
  void onPlayerStateChanged (KPlayerProcess::State);
  /** Receives the progressChanged signal from KPlayerProcess.
    */
  void onPlayerProgressChanged (float);
  /** Receives the infoAvailable signal from KPlayerProcess.
    */
  void onPlayerInfoAvailable (void);

  /** Changes the statusbar test temporarily, used to display a hint on the currently highlighted action.
   */
  void setStatusMessage (const QString&);
  /** Clears the temporary statusbar text when the action is no longer highlighted.
    */
  void clearStatusMessage (void);

  /** Handles widget resized event.
    */
  void onWidgetResized (void);

  /** Handles bar orientation change event. */
//void onBarOrientationChanged (Orientation);
  /** Handles bar place change event. */
//void onBarPlaceChanged (QDockWindow::Place);
  /** Handles bar visibility change event. */
//void onBarVisibilityChanged (bool);
};

/** The KPlayer application class.
  *@author kiriuja
  */
class KPlayerApplication : public KUniqueApplication
{
  Q_OBJECT 

public:
  /** Constructs the KPlayer application.
    */
  KPlayerApplication (void)
    : KUniqueApplication (true, true, true) { }
  /** KPlayer application destructor. Does nothing.
    */
  virtual ~KPlayerApplication() { }

  /** Creates the main window on the first invokation. Calls KPlayer::start().
    */
  virtual int newInstance (void);
};

#endif
