/***************************************************************************
                          kplayerwidget.h
                          ---------------
    begin                : Sun Dec 01 2002
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef KPLAYERWIDGET_H
#define KPLAYERWIDGET_H

#include <qwidget.h>

#include "kplayerprocess.h"

/** The KPlayer widget where mplayer video output is displayed.
  *@author kiriuja
  */
class KPlayerWidget : public QWidget
{
   Q_OBJECT

public: 
  /** The widget constructor. Initializes internal data structures.
      Parameters are passed to the QWidget constructor. */
  KPlayerWidget (QWidget* parent = 0, const char* name = 0);
  /** The widget destructor. Frees allocated memory.
    */
  virtual ~KPlayerWidget();

  /** Returns a pointer to the KPlayerProcess object.
    */
  KPlayerProcess* kPlayerProcess (void) const
    { return m_process; }

  /** Returns whether the widget maintains the original video aspect ratio.
    */
  bool originalAspect (void) const
    { return m_original_aspect; }
  /** Returns whether the widget maintains the video aspect ratio.
    */
  bool isFixedAspect (void) const
    { return (! m_aspect.isEmpty() || m_original_aspect) && ! kPlayerProcess() -> size(). isEmpty(); }
  /** Returns the fixed video aspect ratio in the by-reference parameters.
      The return value is from the previous function.
    */
  bool getFixedAspect (int&, int&);
  /** Returns the current video size.
    */
  const QSize& videoSize (void) const
    { return m_size; }
  /** Sets the video size. Adjusts the given size to the current aspect ratio.
   *  @param size the base video size
   *  @param canGrow whether the size can grow beyond the base size
   *  @param horizontally whether the size should be adjusted horizontally or vertically
    */
  void setVideoSize (const QSize&, bool canGrow = true, bool horizontally = false);
  /** Returns whether the current video size is the given factor of the original size.
    */
  bool isZoomFactor (int, int = 1) const;

  /** Adjusts the given size to the current aspect ratio.
   *  @param size the video size
   *  @param canGrow whether the size can grow beyond the given size
   *  @param horizontally whether the size should be adjusted horizontally or vertically
    */
  void adjustAspect (QSize& size, bool canGrow = false, bool horizontally = false) const;
  /** Adjusts the width of the given size to the current aspect ratio.
   *  @param size the video size
    */
  void adjustWidth (QSize&) const;
  /** Adjusts the height of the given size to the current aspect ratio.
   *  @param size the video size
    */
  void adjustHeight (QSize&) const;

  /** Handles resizing done by the window manager.
   *  @param resizing true if resizing has started, false if resizing has completed
    */
  void resizeHandler (bool resizing);

protected:
  /** The pointer to the KPlayerProcess object.
    */
  KPlayerProcess* m_process;
  /** The maintain original video aspect ratio option.
    */
  bool m_original_aspect;
  /** The currently enforced aspect ratio. Empty if no ratio is forced.
    */
  QSize m_aspect;
  /** The current widget size.
    */
  QSize m_size;
  // Following should be private
  /** Window manager is resizing the top level window.
    */
  bool m_resizing;

signals:
  /** Emitted when the widget is resized.
    */
  void resized (void);

public slots:
  /** Displays the file open dialog and lets the user choose a file.
      Returns true if the user chooses a file, otherwise returns false. */
  bool openFile (void);
  /** Displays the URL requester dialog and lets the user enter a URL.
      Returns true if the user enters a URL, otherwise returns false. */
  bool openUrl (void);
  /** Displays the file open dialog and lets the user choose a subtitle file.
      Returns true if the user chooses a file, otherwise returns false. */
  bool openSubtitle (void);
  /** Displays the URL requester dialog and lets the user enter a subtitle URL.
      Returns true if the user enters a URL, otherwise returns false. */
  bool openSubtitleUrl (void);

  /** Enlarges the video by 50% of the original size.
    */
  void zoomIn (void);
  /** Shrinks the video by 50% of the original size.
    */
  void zoomOut (void);
  /** Sets the video size to the given factor of the original size.
    */
  void setZoomFactor (int, int = 1);

  /** Maintains the original video aspect ratio.
    */
  void maintainOriginalAspect (bool);
  /** Maintains the current video aspect ratio.
    */
  void maintainCurrentAspect (bool);
  /** Sets a fixed video aspect ratio. To reset pass (0, 0).
    */
  void setFixedAspect (int, int);

  /** Processes the widget events. Passes them on to QWidget. */
//virtual bool event (QEvent*);
  /** Processes the widget close event. Stops the KPlayerProcess.
    */
  virtual void closeEvent (QCloseEvent*);
  /** Processes the widget resize event. Ensures that MPlayer
      continues to display video. Emits resized signal. */
  virtual void resizeEvent (QResizeEvent*);

  /** Receives the stateChanged signal from KPlayerProcess.
    */
  void playerStateChanged (KPlayerProcess::State);

  /** Handles the widget resizing. */
//virtual void resize (int, int);
  /** Handles the widget resizing. Calls the above function. */
//virtual void resize (const QSize& size)
//  { resize (size.width(), size.height()); }
  /** Handles the widget geometry change. */
//virtual void setGeometry (int, int, int, int);
};

class KPlayerWorkspace : public QWidget
{
   Q_OBJECT

public: 
  /** The workspace constructor. Creates the KPlayerWidget.
      Parameters are passed to the QWidget constructor. */
  KPlayerWorkspace (QWidget* parent = 0, const char* name = 0);
  /** The workspace destructor. Does nothing.
    */
  virtual ~KPlayerWorkspace() { }

  /** Returns a pointer to the KPlayerWidget object.
    */
  KPlayerWidget* kPlayerWidget (void) const
    { return m_widget; }
  /** Returns a pointer to the KPlayerProcess object.
    */
  KPlayerProcess* kPlayerProcess (void) const
    { return m_widget -> kPlayerProcess(); }

  /** Returns whether the workspace is in free resize mode. Initially set to true.
    */
  bool freeResizeMode (void)
    { return m_free_resize_mode; }
  /** Sets or resets the free resize mode.
    */
  void setFreeResizeMode (bool mode)
    { m_free_resize_mode = mode; }

protected:
  /** The pointer to the KPlayerWidget object.
    */
  KPlayerWidget* m_widget;
  /** The free resize mode setting.
    */
  bool m_free_resize_mode;

public slots:
  /** Resizes the widget to the video size.
    */
  void setVideoSize (void);
  /** Processes the widget resize event. Resizes the KPlayerWidget to the video size.
    */
  virtual void resizeEvent (QResizeEvent*);
};

#endif

// --build=i386-linux --host=i386-linux --target=i386-linux
